#!PERL
#       showdbm - debug program to show entries in DBM file
#
#	$RCSfile: showdbm.pl,v $
#	$Revision: 0.12 $
#	$Author: marten $
#	$Date: 1993/08/06 07:56:57 $
#

dbmopen(%a, $ARGV[0], 0666);

while (($k, $v) = each %a) {
	@tmp = split(/\,/, $v);
	printf "%s - %d (%s)\n", $k, $#tmp+1, $v;
}
