#	$RCSfile: defaults.pl,v $
#	$Revision: 0.13 $
#	$Author: marten $
#	$Date: 1994/09/20 11:46:45 $
# 
# defaults.pl - some default configuration definitions. Will be read
# before the configuration file is read, and the config file will overwrite
# these values when defined.

$PIDFILE   = "/tmp/whoisd.pid";
$CLEANLOCK = "/tmp/cleandb.lock";
$TMPDIR    = "/tmp";
$MAILCMD   = "/usr/lib/sendmail -t";
$DEFMAIL   = "ripe-dbm@ripe.net";
$MAXHITS   = 100;
