#	$RCSfile: adderror.pl,v $
#	$Revision: 1.1 $
#	$Author: marten $
#	$Date: 1994/09/20 11:19:46 $

sub adderror {
    local(*object, $msg) = @_;
    if ($object{"ue"}) {
	$object{"ue"} .= "\n";
    }
    $object{"ue"} .= $msg;
}

sub addwarning {
    local(*object, $msg) = @_;
    if ($object{"uw"}) {
	$object{"uw"} .= "\n";
    }
    $object{"uw"} .= $msg;
}


sub haserror {
    local(*object) = @_;
    return 1 if $object{"ue"};
    return 0;
}

sub haswarning {
    local(*object) = @_;
    return 1 if $object{"uw"};
    return 0;
}

sub rmerrors {
    local(*object) = @_;
    delete $object{"ue"};
    return;
}

sub rmwarnings {
    local(*object) = @_;
    delete $object{"uw"};
    return;
}

1;
