
sub timer {
    local($string, $start) = @_;
    local($tmp1, $tmp2);
    
    if ($start) {
	($tmpuser{$string}, $tmpsystem{$string}, $tmp1, $tmp2) = times;
    } else {
	local($tmpendu{$string}, $tmpends{$string}, $tmp1, $tmp2) = times;
	$timers{$string} += $tmpendu{$string} - $tmpuser{$string};
	$timers{$string} += $tmpends{$string} - $tmpsystem{$string};
    }
}

1;
