#       dbclose - close a database
#
#	$RCSfile: dbclose.pl,v $
#	$Revision: 0.14 $
#	$Author: marten $
#	$Date: 1994/09/20 11:39:22 $

sub dbclose {

        local(*db) = @_;

        if ($db[0] == 1) {
                dbmclose(db);
                close(db);
                $db[0] = 0;

		# No longer valid because of seperate open/close
		# dbmclose(mspnxl);
		# %mspnxl = ();
        }
	$lockcounter = 0;
}

1;
