#       entype - return type of RIPE database entry
#
#	$RCSfile: entype.pl,v $
#	$Revision: 0.13 $
#	$Author: marten $
#	$Date: 1994/09/20 12:13:05 $
#
#       This routine returns the type of RIPE database entry given to it.
#	An entry type is defined by the first line of an entry eg
#	"pn", "in", "rp" etc.
#
#	Arguments:
#	*object		pointer to database entry in assoc array.

sub entype {

    local(*object) = @_;
  
    foreach (keys %OBJATSQ) {
	if ($object{$_}) {
	    return($_);
	}
    }
    return 0;
}
1;
